<?php
// Copyright (C) <2015-present>  <it-novum GmbH>
//
// This file is licensed under the terms of the openITCOCKPIT Enterprise Edition license agreement.
// The license agreement and license key were sent with the order confirmation.

declare(strict_types=1);

namespace SlackModule\Command;

use App\Model\Table\ProxiesTable;
use App\Model\Table\SystemsettingsTable;
use Cake\Console\Arguments;
use Cake\Console\Command;
use Cake\Console\ConsoleIo;
use Cake\Console\ConsoleOptionParser;
use Cake\ORM\TableRegistry;
use GuzzleHttp\Client;
use itnovum\openITCOCKPIT\Exceptions\HostNotFoundException;
use itnovum\openITCOCKPIT\Exceptions\ServiceNotFoundException;
use SlackModule\Lib\SlackData;
use SlackModule\Model\Table\SlackSettingsTable;

/**
 * SlackNotification command.
 */
class SlackNotificationCommand extends Command {

    private $slackSettings;
    private $proxySettings;

    /**
     * @param ConsoleOptionParser $parser
     * @return ConsoleOptionParser
     */
    public function buildOptionParser(ConsoleOptionParser $parser): ConsoleOptionParser {
        /** @var ConsoleOptionParser $parser */
        $parser = parent::buildOptionParser($parser);

        $parser->addOptions([
            'type'             => ['short' => 't', 'help' => __d('oitc_console', 'Type of the notification host or service')],
            'notificationtype' => ['help' => __d('oitc_console', 'Notification type of monitoring engine')],
            'hostname'         => ['help' => __d('oitc_console', 'Host uuid you want to send a notification')],
            'hoststate'        => ['help' => __d('oitc_console', 'current host state')],
            'hostaddress'      => ['help' => __d('oitc_console', 'host address')],
            'output'           => ['help' => __d('oitc_console', 'output')],
            'servicedesc'      => ['help' => __d('oitc_console', 'Service uuid you want to notify')],
            'servicestate'     => ['help' => __d('oitc_console', 'service state')],
        ]);

        return $parser;
    }

    /**
     * @param Arguments $args
     * @param ConsoleIo $io
     * @throws HostNotFoundException
     * @throws ServiceNotFoundException
     */
    public function execute(Arguments $args, ConsoleIo $io) {
        /** @var SlackSettingsTable $SlackSettingsTable */
        $SlackSettingsTable = TableRegistry::getTableLocator()->get('SlackModule.SlackSettings');
        /** @var ProxiesTable $ProxiesTable */
        $ProxiesTable = TableRegistry::getTableLocator()->get('Proxies');
        /** @var SystemsettingsTable $SystemsettingsTable */
        $SystemsettingsTable = TableRegistry::getTableLocator()->get('Systemsettings');
        /** @var SlackData $SlackData */
        $SlackData = new SlackData();

        $this->proxySettings = $ProxiesTable->getSettings();
        $this->slackSettings = $SlackSettingsTable->find()->first()->toArray();

        if ($args->getOption('type') !== '') {
            if ($args->getOption('type') === 'Host') {
                $notification = [
                    'host_uuid'        => $args->getOption('hostname'),
                    'host_state'       => (int)$args->getOption('hoststate'),
                    'host_address'     => $args->getOption('hostaddress'),
                    'notificationtype' => $args->getOption('notificationtype'),
                    'type'             => $args->getOption('type'),
                    'output'           => $args->getOption('output'),
                ];
            }

            if ($args->getOption('type') === 'Service') {
                $notification = [
                    'host_uuid'        => $args->getOption('hostname'),
                    'host_state'       => (int)$args->getOption('hoststate'),
                    'host_address'     => $args->getOption('hostaddress'),
                    'service_uuid'     => $args->getOption('servicedesc'),
                    'service_state'    => $args->getOption('servicestate'),
                    'notificationtype' => $args->getOption('notificationtype'),
                    'type'             => $args->getOption('type'),
                    'output'           => $args->getOption('output'),
                ];
            }
        }

        $acknowledgeValues = 'acknowledged||' . $notification['type'] . '||' . $notification['host_uuid'];

        $systemAddress = $SystemsettingsTable->find()
            ->where([
                'key' => 'SYSTEM.ADDRESS'
            ])->first()->toArray()['value'];

        $output = sprintf("Output: %s\n", $notification['output']);

        if ($notification['type'] == "Host") {
            $host = $SlackData->getHost($notification['host_uuid']);

            if ($notification['host_state'] == "0")
                $state = "UP";
            else if ($notification['host_state'] == "1")
                $state = "DOWN";
            else if ($notification['host_state'] == "2")
                $state = "UNREACHABLE";
            else
                $state = "Unknown";

            $hosturl = sprintf('https://%s/#!/hosts/browser/%s', $systemAddress, $host['id']);
            $notificationText = sprintf("\n\nHost <%s|*%s*> is *%s*\n\n", $hosturl, $host['name'], $state);

            if ($state == "UP" || $this->slackSettings['two_way'] == "0") {
                //$notificationText = sprintf("\nHost *%s* is *%s*\n\n",$host['name'],$state);
                $notificationText .= $output;
            }

            //$notificationText .= sprintf("Time: %s\n",date('H:i:s - d.m.Y T'));
            //$notificationText .= sprintf("Host Name: *%s*\n",$host['name']);
            //$notificationText .= sprintf("Hostaddress: %s\n",$notification['host_address']);
            //$notificationText .= sprintf("State: *%s*\n",$state);
        }

        if ($notification['type'] == "Service") {
            $acknowledgeValues .= '||' . $notification['service_uuid'];
            $host = $SlackData->getHost($notification['host_uuid']);
            $service = $SlackData->getService($notification['service_uuid']);

            $hosturl = sprintf('https://%s/#!/hosts/browser/%s', $systemAddress, $host['id']);
            $serviceurl = sprintf('https://%s/#!/services/browser/%s', $systemAddress, $service['id']);

            if ($notification['service_state'] == "0")
                $state = "OK";
            else if ($notification['service_state'] == "1")
                $state = "WARNING";
            else if ($notification['service_state'] == "2")
                $state = "CRITICAL";
            else
                $state = "UNKNOWN";

            $notificationText = sprintf("\n\nService <%s|*%s*> on Host <%s|*%s*> is *%s*\n\n", $serviceurl, $service['name'], $hosturl, $host['name'], $state);

            if ($state == "OK" || $this->slackSettings['two_way'] == "0") {
                //$notificationText = sprintf("\nService *%s* on Host *%s* is *%s*\n\n",$service['name'],$host['name'],$state);
                $notificationText .= $output;
            }

            //$notificationText .= sprintf("Time: %s\n",date('H:i:s - d.m.Y T'));
            //$notificationText .= sprintf("Service Name: *%s*\n",$service['name']);
            //$notificationText .= sprintf("Hostaddress: %s\n",$notification['host_address']);
            //$notificationText .= sprintf("State: *%s*\n",$state);
        }


        if ($state == "UP" || $state == "OK") {
            $notificationcolor = "#5cb85c";
        }
        if ($state == "CRITICAL" || $state == "DOWN") {
            $notificationcolor = "#d9534f";
        }
        if ($state == "WARNING") {
            $notificationcolor = "#f0ad4e";
        }
        if ($state == "UNKNOWN" || $state == "UNREACHABLE") {
            $notificationcolor = "#adadad";
        }

        if ($state != "UP" && $state != "OK" && $this->slackSettings['two_way'] == "1") {
            $data_string = [
                'text'        => $notificationText,
                'attachments' => [
                    [
                        'text'            => $output,
                        'fallback'        => 'No output',
                        'color'           => $notificationcolor,
                        'callback_id'     => 'host_output',
                        'attachment_type' => 'default',
                    ],
                    [
                        'text'            => "Click to be the first acknowledger for this",
                        'fallback'        => 'You are unable to acknowledge this',
                        'color'           => $notificationcolor,
                        'callback_id'     => 'ack_host',
                        'attachment_type' => 'default',
                        'actions'         => [
                            [
                                'name'  => 'ack_button',
                                'text'  => 'Acknowledge',
                                'style' => 'danger',
                                'type'  => 'button',
                                'value' => $acknowledgeValues,
                            ]
                        ]
                    ]
                ]
            ];
        } else {
            $data_string = [
                'text'        => "",
                'attachments' => [
                    [
                        'text'            => $notificationText,
                        "mrkdwn_in"       => ["text"],
                        'color'           => $notificationcolor,
                        'callback_id'     => 'info',
                        'attachment_type' => 'default'
                    ]
                ]
            ];
        }

        $options = [
            'json'    => $data_string,
            'headers' => [
                'Accept' => 'application/json'
            ]
        ];

        if ($this->slackSettings['use_proxy']) {
            $options['proxy'] = [
                'http'  => sprintf('%s:%s', $this->proxySettings['ipaddress'], $this->proxySettings['port']),
                'https' => sprintf('%s:%s', $this->proxySettings['ipaddress'], $this->proxySettings['port'])
            ];
        } else {
            $options['proxy'] = [
                'http'  => false,
                'https' => false
            ];
        }

        $guzzleclient = new Client();
        $guzzleclient->post($this->slackSettings['api_url'], $options);
    }
}
